﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GSpreadCOM.Domain;

namespace GSpreadCOM
{
	public class Borders : Microsoft.Office.Interop.Excel.Borders
	{
		protected Range range;
		protected Dictionary<Microsoft.Office.Interop.Excel.XlBordersIndex, Border> borders;

		public Borders(Range range)
		{
			this.range = range;
			this.borders = new Dictionary<Microsoft.Office.Interop.Excel.XlBordersIndex, Border>();
			foreach (Microsoft.Office.Interop.Excel.XlBordersIndex index in Enum.GetValues(typeof(Microsoft.Office.Interop.Excel.XlBordersIndex)))
			{
				if (index == Microsoft.Office.Interop.Excel.XlBordersIndex.xlDiagonalDown || index == Microsoft.Office.Interop.Excel.XlBordersIndex.xlDiagonalUp)
				{
					continue;
				}
				this.borders.Add(index, new Border(this, index));
			}
		}

		#region service members

		public void SetBorders(bool? top, bool? left, bool? bottom, bool? right, bool? vertical, bool? horizontal, string htmlColor, string lineStyle)
		{
			range.ScriptService.call("setBorders",
				range,
				
				top,
				left,
				bottom,
				right,
				vertical,
				horizontal,
				htmlColor,
				lineStyle
			);
		}

		public void UpdateBorders()
		{
			var grouped = borders.Values.GroupBy(b => new { b.LineStyle, b.Color });
			foreach (var group in grouped)
			{
				var groupKey = group.Key;

				bool? update = groupKey.LineStyle != Microsoft.Office.Interop.Excel.XlLineStyle.xlLineStyleNone;

				bool? top = null;
				bool? left = null;
				bool? bottom = null;
				bool? right = null;
				bool? vertical = null;
				bool? horizontal = null;

				foreach (Border border in group)
				{
					if (!border.IsChanged) continue;

					if (border.Index == Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeTop) top = update;
					else if (border.Index == Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeLeft) left = update;
					else if (border.Index == Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeBottom) bottom = update;
					else if (border.Index == Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight) right = update;
					else if (border.Index == Microsoft.Office.Interop.Excel.XlBordersIndex.xlInsideVertical) vertical = update;
					else if (border.Index == Microsoft.Office.Interop.Excel.XlBordersIndex.xlInsideHorizontal) horizontal = update;
				}

				SetBorders(
					top,
					left,
					bottom,
					right,
					vertical,
					horizontal,
					ExcelColorPalette.IntToHtmlColor(groupKey.Color),
					((Microsoft.Office.Interop.Excel.XlLineStyle) groupKey.LineStyle).ToString()
				);
			}
		}

		#endregion

		/// <summary>
		/// Returns an Application object that represents the creator of the specified object. Read-only.
		/// </summary>
		public Microsoft.Office.Interop.Excel.Application Application
		{
			get { return range.Application; }
		}

		/// <summary>
		/// Returns or sets the primary color of the object. Read/write.
		/// </summary>
		public dynamic Color
		{
			get
			{
				dynamic result = borders.First().Value.Color;
				foreach (Border border in borders.Values)
				{ 
					if (border.Color != result)
					{
						return ExcelColor.FAULT_VALUE;
					}
				}
				return result;
			}
			set
			{
				var color = new ExcelColor(value);
				foreach (Border border in borders.Values)
				{
					border.SetColor(color);
				}
				UpdateBorders();
			}
		}

		/// <summary>
		/// Returns or sets an int value that represents the color of all four borders.
		/// </summary>
		public dynamic ColorIndex
		{
			get
			{
				dynamic result = borders.First().Value.ColorIndex;
				foreach (Border border in borders.Values)
				{ 
					if (border.ColorIndex != result)
					{
						return ExcelColor.FAULT_VALUE;
					}
				}
				return result;
			}
			set
			{
				var color = ExcelColorPalette.GetByIndex(value);
				foreach (Border border in borders.Values)
				{
					border.SetColor(color);
				}
				UpdateBorders();
			}
		}

		/// <summary>
		/// Returns an int value that represents the number of objects in the collection.
		/// </summary>
		public int Count
		{
			get
			{
				return borders.Count;
			}
		}

		/// <summary>
		/// Returns or sets the line style for the borders. Read/write XlLineStyle.
		/// </summary>
		public dynamic LineStyle
		{
			get
			{
				dynamic result = borders.First().Value.LineStyle;
				foreach (Border border in borders.Values)
				{ 
					if (border.LineStyle != result)
					{
						return null;
					}
				}
				return result;
			}
			set
			{
				if (value is Microsoft.Office.Interop.Excel.XlLineStyle)
				{
					Microsoft.Office.Interop.Excel.XlLineStyle style = value;
					foreach (Border border in borders.Values)
					{
						border.SetLineStyle(style);
					}
					UpdateBorders();
				}
			}
		}

		/// <summary>
		/// Synonym for Borders.LineStyle.
		/// </summary>
		public dynamic Value
		{
			get
			{
				return LineStyle;
			}
			set
			{
				LineStyle = value;
			}
		}

		/// <summary>
		/// Returns an int value that represents the weight of all borders. Read-only.
		/// </summary>
		public dynamic Weight
		{
			get
			{
				dynamic result = borders.First().Value.Weight;
				foreach (Border border in borders.Values)
				{ 
					if (border.Weight != result)
					{
						return Border.FAULT_WEIGHT;
					}
				}
				return result;
			}
			set
			{
				foreach (Border border in borders.Values)
				{
					border.Weight = value;
				}
			}
		}

		Microsoft.Office.Interop.Excel.Border Microsoft.Office.Interop.Excel.Borders.get_Item(Microsoft.Office.Interop.Excel.XlBordersIndex Index)
		{
			return this[Index];
		}

		/// <summary>
		/// Returns a Border object that represents one of the borders of either a range of cells or a style.
		/// </summary>
		/// <param name="Index">XlBordersIndex</param>
		/// <returns>Border</returns>
		public Microsoft.Office.Interop.Excel.Border this[Microsoft.Office.Interop.Excel.XlBordersIndex Index]
		{
			get
			{
				return borders[Index];
			}
		}

		#region unused members

		public Microsoft.Office.Interop.Excel.XlCreator Creator
		{
			get { throw new NotImplementedException(); }
		}

		public System.Collections.IEnumerator GetEnumerator()
		{
			throw new NotImplementedException();
		}

		public dynamic Parent
		{
			get { throw new NotImplementedException(); }
		}

		public dynamic ThemeColor
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public dynamic TintAndShade
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		#endregion
	}
}
